var fs = require("fs");
var book = require('topsin.excelxs');
var db = require('topsin.database');
var error = require('topsin.error');


function ExlDb(props) {
    this.props = props
}

ExlDb.prototype.read = function (path, suffix) { // 读取方法 接收参数1 string 文件夹路径 参数2 array 读取对应后缀的文件
    var suffix = suffix || "XLSX"
    var files = fs.listDir(path).filter(function (v) {
        return Array.isArray(suffix) ? suffix.indexOf(v.completeSuffix) >= 0 :
            v.completeSuffix === suffix
    })
    if (files.length === 0) { return }
    files = files.map(function (v) { return (v.dir + '/' + v.name) })
    return files;
}

ExlDb.prototype.readExl = function (path, config) {  // 读取列表 根据配置获取数据
    if (book.open(path)) {
        var sheet = book.getActiveSheet()  // 获取当前表
        var lastRow = sheet.getLastRow()  // 获取有内容的最后一行
        var keysCount = sheet.getLastCol()  // 字段数量
        var res = []  // 存放结果
        var KeyConfig = []  // 存放配置信息
        this.dbkeys = []  // 存放数据库字段名
        for (var i = 0; i < keysCount; i++) {
            KeyConfig[i] = config[getColKey(i + 1)]
            if(!KeyConfig[i]["attr"]){
                this.dbkeys.push(KeyConfig[i]["dbKey"])
            }
        }
        this.dbkeys.push("attr_data")
        for (var i = 1; i < lastRow; i++) {  // 双重循环 根据配置信息获取数据
            res[i-1] = {}
            res[i-1]["attr_data"] = {}
            for(var j = 0; j < keysCount; j++){
                if(KeyConfig[j]["attr"]){
                    res[i-1]["attr_data"][KeyConfig[j]["dbKey"]] = KeyConfig[j]["format"] ?
                    KeyConfig[j]["format"](sheet.getCellValue(i, j)) : sheet.getCellValue(i, j)
                } else {
                    res[i-1][KeyConfig[j]["dbKey"]] = KeyConfig[j]["format"] ?
                    KeyConfig[j]["format"](sheet.getCellValue(i, j)) : sheet.getCellValue(i, j)
                }
            }
        }
        book.close()
        return res
    }
}
ExlDb.prototype.openDb = function(dbConfig){
    db.addConnection(dbConfig.connect, dbConfig.connectName);  // 连接数据库
    this.tablename = dbConfig.tableName  // 设置数据库表名
    this.connectName = dbConfig.connectName 
    this.query = db.query(dbConfig.connectName);  // 获取操作对象
    return this
}
ExlDb.prototype.closeDb = function(){  // 关闭数据库连接
    db.removeConnection(this.connectName);
}
ExlDb.prototype.dbGetAll = function(){  // 获取数据库所有数据
    this.query.begin();
    var data;
    try {
        data = this.query.selectArrayMap({
            table: this.tablename,
            field: '*',
            field_format:{"tags":'array', attr_data:'json'}
        });
        if (this.query.lastError().isValid()){
            throw this.query.lastError().text();
        }
        this.query.commit();
    } catch (error) {
        print("获取数据库数据错误", error);
        this.query.rollback();
    }
    return data
}
ExlDb.prototype.dbDeleteAll = function(){ // 删除所有数据
    return this.query.deleteRow({
        table:this.tablename,
        where: ["id >= 0"]
    })
}

ExlDb.prototype.insertAny = function(datas){ // 批量插入数据
    console.log(this.dbkeys);
    
    this.query.begin()
    try{
        this.query.batchInsert(
            this.tablename,
            this.dbkeys,
            datas
        );
        if (this.query.lastError().isValid()){
            throw this.query.lastError().text();
        }
        this.query.commit();
    }
    catch(err) {
        print("插入数据错误", err)
        this.query.rollback();
    }
}

ExlDb.prototype.UpdateKeys = function(data, id){  // 更新数据
    this.query.begin()
    try {
        this.query.updateRow({
            table:this.tablename,
            data:data,
            where:{id:id},
            update_policy:{tags:'array_append', attr_data:'json_merge'}
        })
        if (this.query.lastError().isValid()){
            throw this.query.lastError().text();
        }
        this.query.commit();
    } catch (error) {
        print("更新错误", err)
        this.query.rollback();
    }
}

module.exports = function (props) {
    return new ExlDb(props);
}




// 数字转字母
function getColKey(num) {
    var stringArray = [];
    function numToString(numm) {
        stringArray.length = 0;
        var numToStringAction = function (nnum) {
            var num = nnum - 1;
            var a = parseInt(num / 26);
            var b = num % 26;
            stringArray.push(String.fromCharCode(64 + parseInt(b + 1)));
            if (a > 0) {
                numToStringAction(a);
            }
        }
        numToStringAction(numm);
        return stringArray.reverse().join("");
    }
    return numToString(num)
}